import os
import uuid
import datetime

import pytest

from utf_queue_client.clients.sqa_test_result_producer import (
    DummySqaTestResultProducer,
    SqaTestResultProducer,
    SqaTestResultProducerFactory,
)


def test_result_producer_empty_url():
    with pytest.raises(RuntimeError):
        _ = SqaTestResultProducer()


@pytest.mark.parametrize("queue_consumer", [60, 120], indirect=True)
@pytest.mark.parametrize("use_http_producer", [True, False])
def test_result_producer_central_queue(
    request,
    sqa_app_build_result,
    sqa_test_result,
    sqa_test_session,
    amqp_url,
    queue_consumer,
    use_http_producer,
):
    os.environ["UTF_PRODUCER_APP_ID"] = request.node.name
    os.environ["UTF_QUEUE_USE_HTTP_PRODUCER"] = "true" if use_http_producer else "false"
    producer = SqaTestResultProducerFactory.create_producer(
        use_http_producer=use_http_producer
    )
    producer.publish_app_build_result(sqa_app_build_result)
    queue_consumer.expect_messages(1)
    producer.publish_test_session_start(sqa_test_session)
    queue_consumer.expect_messages(2)
    producer.publish_test_session_stop(sqa_test_session)
    queue_consumer.expect_messages(3)
    producer.publish_test_result(sqa_test_result)
    queue_consumer.expect_messages(4)


def test_dummy_producer(sqa_app_build_result, sqa_test_result, sqa_test_session):
    producer = DummySqaTestResultProducer()
    producer.publish_app_build_result(sqa_app_build_result)
    producer.publish_test_session_start(sqa_test_session)
    producer.publish_test_session_stop(sqa_test_session)
    producer.publish_test_result(sqa_test_result)


def test_result_producer_dev(
    sqa_app_build_result,
    sqa_test_result,
    sqa_test_session,
):
    os.environ["UTF_PRODUCER_APP_ID"] = "test_result_producer_dev"
    os.environ["UTF_QUEUE_USE_HTTP_PRODUCER"] = "true"
    os.environ["UTF_QUEUE_SERVER_URL"] = (
        "amqps://mwlabuser-jenkins:Lab10Rocks%3f%3f@utf-queue-central.dev.silabs.net:443/%2f"
    )
    producer = SqaTestResultProducerFactory.create_producer(use_http_producer=True)
    producer.queue_name = "sqa_test_results_dev"
    for _ in range(0, 1):
        sqa_test_session.PK_ID = str(uuid.uuid4())
        sqa_test_session.startTime = datetime.datetime.now(tz=datetime.UTC).isoformat()
        sqa_test_session.stopTime = None
        print("Session PKID = ", sqa_test_session.PK_ID)
        sqa_app_build_result.session_pk_id = sqa_test_session.PK_ID

        producer.publish_test_session_start(sqa_test_session)
        sqa_test_session.PASS_cnt = 100
        sqa_test_session.FAIL_cnt = 0
        sqa_test_session.SKIP_cnt = 0
        for i in range(0, 5):
            sqa_app_build_result.test_duration_sec = i
            sqa_app_build_result.app_description = "app description " + str(i + 1)
            producer.publish_app_build_result(sqa_app_build_result)
        sqa_test_session.stopTime = datetime.datetime.now(tz=datetime.UTC).isoformat()
        producer.publish_test_session_stop(sqa_test_session)


# ede5e03b-e8d0-4604-8dd5-ed4c6c5a2686
